package ai

import (
	"bytes"
	"encoding/json"
	"errors"
	"io"
	"net/http"
	"os"

	"github.com/rs/zerolog/log"
	"pluja.dev/kycnot.me/database"
	"pluja.dev/kycnot.me/utils"
)

type AnthropicMessage struct {
	Role    string `json:"role"`
	Content string `json:"content"`
}

type AnthropicMessagePartResponse struct {
	Type string `json:"type"`
	Text string `json:"text"`

	// Optional fields, only present for tools responses
	ID    string                 `json:"id,omitempty"`
	Name  string                 `json:"name,omitempty"`
	Input map[string]interface{} `json:"input,omitempty"`
}

type AnthropicMessageResponse struct {
	ID           string                         `json:"id"`
	Type         string                         `json:"type"`
	Model        string                         `json:"model"`
	Role         string                         `json:"role"`
	Content      []AnthropicMessagePartResponse `json:"content"`
	StopReason   string                         `json:"stop_reason"`
	Stop         string                         `json:"stop"`
	StopSequence string                         `json:"stop_sequence"`
}

func GetTosReviewsClaude(text string) ([]database.TosReview, error) {
	if len(text) < 10 {
		return nil, errors.New("empty text")
	}

	var highlights []database.TosReview

	messages := []AnthropicMessage{
		{
			Role:    "user",
			Content: text,
		},
	}

	resp, err := getResponse(messages)
	if err != nil {
		log.Debug().Err(err).Msgf("Could not get response from OpenAI")
		return highlights, err
	}

	if len(resp.Content) == 0 {
		return highlights, nil
	}

	log.Debug().Msgf("TEXT RESP: \n %s", resp.Content[0].Text)
	highlights, err = jsonToHighlights(resp.Content[0].Text)
	if err != nil {
		log.Debug().Err(err).Msgf("Json error")
		return highlights, err
	}
	return highlights, nil
}

func getResponse(messages []AnthropicMessage) (AnthropicMessageResponse, error) {
	apiKey := os.Getenv("ANTHROPIC_API_KEY")
	model := utils.Getenv("ANTHROPIC_MODEL", "claude-3-sonnet-20240229")

	url := "https://api.anthropic.com/v1/messages"

	data := map[string]interface{}{
		"model":      model,
		"system":     sysPrompt,
		"max_tokens": 4096,
		"messages":   messages,
	}

	jsonData, err := json.Marshal(data)
	if err != nil {
		log.Debug().Err(err).Msgf("Could not marshal data into json")
		return AnthropicMessageResponse{}, err
	}

	req, err := http.NewRequest("POST", url, bytes.NewBuffer(jsonData))
	if err != nil {
		log.Debug().Err(err).Msgf("Request error")
		return AnthropicMessageResponse{}, err
	}

	req.Header.Set("x-api-key", apiKey)
	req.Header.Set("anthropic-version", "2023-06-01")
	req.Header.Set("content-type", "application/json")

	client := &http.Client{}
	resp, err := client.Do(req)
	if err != nil {
		log.Debug().Err(err).Msgf("Request error")
		return AnthropicMessageResponse{}, err
	}
	defer resp.Body.Close()

	body, err := io.ReadAll(resp.Body)
	if err != nil {
		log.Debug().Err(err).Msgf("Body read error")
		return AnthropicMessageResponse{}, err
	}

	var response AnthropicMessageResponse
	err = json.Unmarshal(body, &response)
	if err != nil {
		log.Debug().Err(err).Msgf("Could not unmarshal data into struct")
		return AnthropicMessageResponse{}, err
	}

	return response, nil
}
